@@ -54,15 +54,22 @@ SurfaceInterfacePrivate::SurfaceInterfacePrivate(SurfaceInterface *q)
 
 SurfaceInterfacePrivate::~SurfaceInterfacePrivate()
 {
-    for (KWaylandFrameCallback *frameCallback : current.frameCallbacks) {
+    // Need a copy to avoid hitting invalidated iterators in the for loop.
+    const QList<KWaylandFrameCallback *> currentFrameCallbacks = current.frameCallbacks;
+    for (KWaylandFrameCallback *frameCallback : currentFrameCallbacks) {
         frameCallback->destroy();
     }
-    for (KWaylandFrameCallback *frameCallback : pending.frameCallbacks) {
+
+    const QList<KWaylandFrameCallback *> pendingFrameCallbacks = pending.frameCallbacks;
+    for (KWaylandFrameCallback *frameCallback : pendingFrameCallbacks) {
         frameCallback->destroy();
     }
-    for (KWaylandFrameCallback *frameCallback : cached.frameCallbacks) {
+
+    const QList<KWaylandFrameCallback *> cachedFrameCallbacks = cached.frameCallbacks;
+    for (KWaylandFrameCallback *frameCallback : cachedFrameCallbacks) {
         frameCallback->destroy();
     }
+
     if (current.buffer) {
         current.buffer->unref();
     }
@@ -298,9 +305,6 @@ void SurfaceInterfacePrivate::surface_attach(Resource *resource, struct ::wl_res
     Q_UNUSED(resource)
     pending.bufferIsSet = true;
     pending.offset = QPoint(x, y);
-    if (pending.buffer) {
-        delete pending.buffer;
-    }
     if (!buffer) {
         // got a null buffer, deletes content in next frame
         pending.buffer = nullptr;
@@ -308,7 +312,7 @@ void SurfaceInterfacePrivate::surface_attach(Resource *resource, struct ::wl_res
         pending.bufferDamage = QRegion();
         return;
     }
-    pending.buffer = new BufferInterface(buffer, q);
+    pending.buffer = BufferInterface::get(compositor->display(), buffer);
     QObject::connect(pending.buffer, &BufferInterface::aboutToBeDestroyed, q,
         [this](BufferInterface *buffer) {
             if (pending.buffer == buffer) {
@@ -524,7 +528,6 @@ void SurfaceInterfacePrivate::swapStates(State *source, State *target, bool emit
             if (emitChanged) {
                 target->buffer->unref();
             } else {
-                delete target->buffer;
                 target->buffer = nullptr;
             }
         }
